<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\Repositories;
use Util\Environment;
use Util\Messages;
use Ems\Util\ArrayUtil;

use Ems\Services\ParceiroNegocioService;
use Ems\Repository\OperadorRepository;
use Ems\Repository\ParamOrgRepository;

class OperadorService {

    /** @var OperadorRepository */
    protected $operadorRepository;

    /** @var Environment */
    protected $environment;

    /** @var ParamOrgRepository */
    protected $paramOrgRepository;

    public function __construct(
        EntityManager $entityManager,
        Environment $environment,
        ParceiroNegocioService $parceiroNegocioService,
        OperadorRepository $operadorRepository,
        ParamOrgRepository $paramOrgRepository
    ) {
        $this->entityManager            = $entityManager;
        $this->environment              = $environment;
        $this->parceiroNegocioService   = $parceiroNegocioService;
        $this->operadorRepository       = $operadorRepository;
        $this->paramOrgRepository       = $paramOrgRepository;
    }


    /**
     *  Função que retorna dados úteis do operador.
     *
     *  @param string $cdoperador
     *
     *  @return Array $dadosOperador
     *      'NMOPERADOR'    => $nmoperadornotificacao,
     *      'EMAILOPEADOR'  => $emailoperadornotificacao
     *
     *  @throws Exception
     */
    public function getDadosOperador($cdoperador, $nrorg = null) {
        if (empty($nrorg)) {
            $nrorg = $this->environment->getNrorg();
        }

        $operador = $this->entityManager->getRepository(Repositories::OPERADOR)->findOneBy(array('nrorg' => $nrorg, 'cdoperador' => $cdoperador));

        if (!empty($operador)) {
            $parceiroNegocioOperador = $this->entityManager->getRepository(Repositories::PARCNEGOCIO)->findOneBy(array('nrorg' => $nrorg, 'nrparcnegocio' => $operador->getNrparcnegocio()));

            if (!empty($parceiroNegocioOperador)) {
                $nmoperadornotificacao = $parceiroNegocioOperador->getNmprincipalparc();
                $emailoperadornotificacao = $this->parceiroNegocioService->getEmailParcNegocio($operador->getNrparcnegocio());
            } else {
                throw new \Exception(Messages::PARCNEGOCIO_OPERADOR_NOT_FOUND);
            }
        } else {
            throw new \Exception(Messages::OPERADOR_NOT_FOUND);
        }

        $dadosOperador = array(
            'NMOPERADOR' => $nmoperadornotificacao,
            'EMAILOPERADOR' => $emailoperadornotificacao
        );

        return $dadosOperador;
    }

    public function getPerfisDeAcesso(
        string $cdOperador
    ): array {
        $nrOrg = (int) $this->environment->getNrorg();

        $params = [
            'P_NRORG'      => $nrOrg,
            'P_CDOPERADOR' => $cdOperador
        ];

        $perfis = $this->operadorRepository->getPerfisDeAcesso($params);

        $nrPerfis = array_map(function($perfil) {
            return $perfil['NRPERFILACESSO'];
        }, $perfis);

        return $nrPerfis;
    }

    public function operadorIsDP(
        string $cdOperador
    ): bool {
        $nrOrg = (int) $this->environment->getNrorg();

        $nrPerfis = $this->getPerfisDeAcesso($cdOperador);

        $paramOrg = $this->paramOrgRepository->getParamOrg($nrOrg);

        $stringPerfisDP = $paramOrg['NRPERFACESSADM'];
        $stringPerfisDP = is_null($stringPerfisDP) ? '' : $stringPerfisDP;
        
        $perfisDP = ArrayUtil::fromStringSQL($stringPerfisDP, ArrayUtil::COMMA_ARRAY_STRING_SEPARATOR, false);
        
        $intersection = ArrayUtil::intersection($nrPerfis, $perfisDP);

        $isDP = count($intersection) > 0;

        return $isDP;
    }
    
    public function getOperadoresDp(){
        $nrorg = $this->environment->getNrorg();
        return $this->operadorRepository->getOperadoresDp($nrorg);
    }
}