/* Sistemas Integrados TECNISA
   Servios

Nome: INCLUSAO_DE_DUPLICATA

Descrio:
Gerao do boleto real de contas a receber referentes a uma duplicata que est
sendo includa.

P_CDFILIAL    - Cdigo da Filial
P_NRLANCTODUP - Nmero da Duplicata
P_NRLANBOLREC - Nmero do novo boleto
P_IDRESULT    - Identificador de resultado
                'N' - Ok.
                'A' - Contas (vencimentos) j agrupados em outro(s) boleto(s) no sistema de Contas a Receber.
                'B' - No existem contas associadas  duplicata.
                'C' - Nmero do boleto (interno) no informado.
                'G' - Boletos j baixados
                'H' - Contas ou boletos parcelados
                'K' - Conta prorrogada - (DESATIVADO)

Para aplicar e debugar
-- @C:\WIN_V4\SQL\END_CONV\ORACLE\1010_FINSTPO.SQL;
-- EXECUTE INCLUSAO_DE_DUPLICATA('9999', '0000000000', '0000000000', :P_IDRESULT)
-- SET SERVEROUTPUT ON
-- DBMS_OUTPUT.PUT_LINE('xx');

Criao:
13/01/00 - Ederson

*/

CREATE OR REPLACE
PROCEDURE INCLUSAO_DE_DUPLICATA
  (P_CDFILIAL    IN VARCHAR2,
   P_NRLANCTODUP IN VARCHAR2,
   P_NRLANBOLREC IN BOLETRECEBER.NRLANBOLREC%TYPE,
   P_IDRESULT    IN OUT VARCHAR2) IS

   V_CTDTVENBOL   NUMBER(4);
   V_DTVENCDUP    DUPLICATA.DTVENCDUP%TYPE;
   V_CDEMPRESA    DUPLICATA.CDEMPRESA%TYPE;
   V_CDMOTIPROR   GERAL_REC.CDMOTIPROR%TYPE;
   V_CDCLIENTE    CLIENTE.CDCLIENTE%TYPE;
   V_CDTPBAIXAREC BOLETRECEBER.CDTPBAIXAREC%TYPE;
   V_CDREDUCONT   BOLETRECEBER.CDREDUCONT%TYPE;
   V_CTTPRECBOL   NUMBER;
   V_CTREDUCONT   NUMBER;
   V_VRBOLETO     NUMBER;
   V_PEMULTA      NUMBER;
   V_PEMORA       NUMBER;
   V_NRREGFOR     NUMBER;
   V_NRBOLETO     NUMBER;
   V_NRDUPLIC     NUMBER;
   V_NRAJDUPLI    NUMBER;
   V_VRAJDUPLI    NUMBER;
   V_CTAJBOL      NUMBER;
   V_NRBOLBAIXADO NUMBER;
   V_NRPARCELA    NUMBER;
   V_NRCNTPROR    NUMBER;
   V_VRMOVFXBOL   MOVFXAJBOLR.VRFXAJBOLREC%TYPE;
   V_NRNOTFISC    VARCHAR2(600);
   V_TXMENSBOL    BOLETRECEBER.TXMENSBOLBOL%TYPE;

   V_NMRAZSOCCLIE CLIENTE.NMRAZSOCCLIE%TYPE;
   V_NMFANTCLIE   CLIENTE.NMFANTCLIE%TYPE;
   V_NMTPBAIXAREC TIPOBAIXAREC.NMTPBAIXAREC%TYPE;
   V_TIPOBAIXA  NUMBER;
   V_CDTPBAIXA TIPOBAIXAREC.CDTPBAIXAREC%TYPE;


BEGIN

   P_IDRESULT := 'N';

   /* - VERIFICA SE O BOLETO POSSUI MAIS DE UMA CONTA ----------------------- */
   SELECT
      COUNT(*)
   INTO
      V_NRREGFOR
   FROM
      CONTARECEBER
   WHERE
      (CDFILIALDUP = P_CDFILIAL)    AND
      (NRLANCTODUP = P_NRLANCTODUP) AND
      (NRLANBOLREC IS NOT NULL);

   IF V_NRREGFOR > 0 THEN
      FOR C_CNTBOL IN
         (SELECT
             CDFILIBOLREC,
             NRLANBOLREC
          FROM
             CONTARECEBER
          WHERE
             (CDFILIALDUP = P_CDFILIAL)    AND
             (NRLANCTODUP = P_NRLANCTODUP) AND
             (NRLANBOLREC IS NOT NULL)
         ) LOOP

         SELECT
            COUNT(*)
         INTO
            V_NRBOLETO
         FROM
            CONTARECEBER
         WHERE
            (CDFILIBOLREC = C_CNTBOL.CDFILIBOLREC) AND
            (NRLANBOLREC  = C_CNTBOL.NRLANBOLREC)  AND
            (NRLANBOLREC IS NOT NULL);

         IF V_NRBOLETO > 1 THEN
            SELECT
               COUNT(DISTINCT NRLANCTODUP)
            INTO
               V_NRDUPLIC
            FROM
               CONTARECEBER
            WHERE
               (CDFILIBOLREC = C_CNTBOL.CDFILIBOLREC) AND
               (NRLANBOLREC  = C_CNTBOL.NRLANBOLREC)  AND
               (NRLANBOLREC IS NOT NULL);

            IF V_NRDUPLIC > 1 THEN
               P_IDRESULT := 'A';
            END IF;
         END IF;
      END LOOP;
   ELSE
      P_IDRESULT := 'B';
   END IF;

   IF P_NRLANBOLREC = '' THEN
      P_IDRESULT := 'C';
   END IF;

   /* - VERIFICA SE O BOLETO J FOI BAIXADO ---------------------------------- */
   SELECT
      COUNT(*)
   INTO
      V_NRBOLBAIXADO
   FROM
      CONTARECEBER CR,
      BOLETRECEBER BO
   WHERE
      (CDFILIALDUP    = P_CDFILIAL)      AND
      (NRLANCTODUP    = P_NRLANCTODUP)   AND
      (BO.CDFILIAL    = CR.CDFILIBOLREC) AND
      (BO.NRLANBOLREC = CR.NRLANBOLREC)  AND
      (BO.NRBAIXAREC IS NOT NULL);

   IF V_NRBOLBAIXADO > 0 THEN
      P_IDRESULT := 'G';
   END IF;

   /* - VERIFICA SE O CONTA FOI PARCELADA ---------------------------------- */
   /*SELECT
      COUNT(DISTINCT NRPARCELAREC)
   INTO
      V_NRPARCELA
   FROM
      CONTARECEBER
   WHERE
      (CDFILIALDUP    = P_CDFILIAL)    AND
      (NRLANCTODUP    = P_NRLANCTODUP) AND
      (NRLANBOLREC IS NOT NULL);

   IF V_NRPARCELA > 1 THEN
      P_IDRESULT := 'H';
   END IF;*/

   /* - VERIFICA SE A CONTA FOI PRORROGADA ----------------------------------
   SELECT
      COUNT(*)
   INTO
      V_NRCNTPROR
   FROM
      CONTARECEBER
   WHERE
      (CDFILIALDUP  = P_CDFILIAL)    AND
      (NRLANCTODUP  = P_NRLANCTODUP) AND
      (DTORIGVENREC <> DTATUAVENREC)  AND
      (NRLANBOLREC IS NOT NULL);

   IF V_NRCNTPROR > 0 THEN
      P_IDRESULT := 'K';
   END IF;
   */
   IF P_IDRESULT = 'N' THEN
      SELECT
         MAX(DTVENCDUP),
         MAX(CDEMPRESA)
      INTO
         V_DTVENCDUP,
         V_CDEMPRESA
      FROM
         DUPLICATA
      WHERE
         (CDFILIAL    = P_CDFILIAL) AND
         (NRLANCTODUP = P_NRLANCTODUP);

      SELECT
         MAX(CDMOTIPROR)
      INTO
         V_CDMOTIPROR
      FROM
         GERAL_REC;

      /* - VERIFICA SE OS BOLETOS POSSUEM A MESMA CONTA E O MESMO TIPO DE
           RECEBIMENTO PARA GRAVAR NO BOLETO QUE SER GERADO --- */
      SELECT
         COUNT(DISTINCT B.CDTPBAIXAREC),
         COUNT(DISTINCT B.CDREDUCONT)
      INTO
         V_CTTPRECBOL,
         V_CTREDUCONT
      FROM
         BOLETRECEBER B,
         CONTARECEBER C
      WHERE
         (B.CDFILIAL    = C.CDFILIBOLREC) AND
         (B.NRLANBOLREC = C.NRLANBOLREC)  AND
         (CDFILIALDUP  = P_CDFILIAL)      AND
         (NRLANCTODUP  = P_NRLANCTODUP);

      IF V_CTTPRECBOL = 1 AND V_CTREDUCONT = 1 THEN
         SELECT
            MAX(B.CDTPBAIXAREC),
            MAX(B.CDREDUCONT)
         INTO
            V_CDTPBAIXAREC,
            V_CDREDUCONT
         FROM
            BOLETRECEBER B,
            CONTARECEBER C
         WHERE
            (B.CDFILIAL    = C.CDFILIBOLREC) AND
            (B.NRLANBOLREC = C.NRLANBOLREC)  AND
            (CDFILIALDUP  = P_CDFILIAL)      AND
            (NRLANCTODUP  = P_NRLANCTODUP);
      ELSE
         V_CDTPBAIXAREC := NULL;
         V_CDREDUCONT   := NULL;
      END IF;

      V_VRBOLETO  := 0;
      V_NRNOTFISC := '';
      FOR C_CONTA IN
         (SELECT
             CR.CDFILIAL,     CR.NRLANCTONF,  CR.NRSEQVENREC, CR.NRPARCELAREC,
             CR.DTATUAVENREC, CR.VRORIGREC,   CR.VRATUAREC,   CR.CDFILIBOLREC,
             CR.NRLANBOLREC,  CR.CDCLASSFINA, CR.CDEMPRESA,   CR.CDFIMOVFLREC,
             CR.DTMOVFLUXO,   CR.IDMOVFLUXO,  NF.NRNOTAFISC,  NVL(CR.QTDIASPREREC, 0) QTDIASPREREC
          FROM
             CONTARECEBER CR,
             NOTAFISCAL   NF
          WHERE
             (CR.CDFILIALDUP = P_CDFILIAL   ) AND
             (CR.NRLANCTODUP = P_NRLANCTODUP) AND
             (NF.CDFILIAL    = CR.CDFILIAL  ) AND
             (NF.NRLANCTONF  = CR.NRLANCTONF) AND
             (CR.NRLANBOLREC IS NOT NULL)
          ) LOOP

         IF V_NRNOTFISC IS NULL THEN
            V_NRNOTFISC := C_CONTA.NRNOTAFISC;
         ELSE
            IF LENGTH(V_NRNOTFISC) < 500 THEN
               V_NRNOTFISC := V_NRNOTFISC || '; ' || C_CONTA.NRNOTAFISC;
            END IF;
         END IF;

         V_VRBOLETO := V_VRBOLETO + C_CONTA.VRATUAREC; /* SOMA O VALOR DAS CONTAS PARA GRAVAR NO BOLETO */

         /* - PRORROGA A CONTA E ATUALIZA O FLUXO DE CAIXA ------------------ */
      	IF C_CONTA.DTATUAVENREC <> V_DTVENCDUP THEN

            ATUALIZA_FLUXO_DE_CAIXA(C_CONTA.CDEMPRESA,
                                    C_CONTA.CDFIMOVFLREC,
                                    C_CONTA.DTMOVFLUXO,
                                    C_CONTA.IDMOVFLUXO,
                                    C_CONTA.CDCLASSFINA,
                                    C_CONTA.VRORIGREC * -1,
                                    0);

            ATUALIZA_FLUXO_DE_CAIXA(C_CONTA.CDEMPRESA,
                                    C_CONTA.CDFIMOVFLREC,
                                    V_DTVENCDUP + TRUNC(C_CONTA.QTDIASPREREC),
                                    C_CONTA.IDMOVFLUXO,
                                    C_CONTA.CDCLASSFINA,
                                    C_CONTA.VRORIGREC,
                                    0);

            UPDATE CONTARECEBER
            SET
               DTATUAVENREC = V_DTVENCDUP,
               CDMOTIPROR   = V_CDMOTIPROR,
               DTMOVFLUXO   = V_DTVENCDUP + TRUNC(C_CONTA.QTDIASPREREC)
            WHERE
               (CDFILIAL     = C_CONTA.CDFILIAL)     AND
               (NRLANCTONF   = C_CONTA.NRLANCTONF)   AND
               (NRSEQVENREC  = C_CONTA.NRSEQVENREC)  AND
               (NRPARCELAREC = C_CONTA.NRPARCELAREC);

            /* -- ATUALIZA O FLUXO DE CAIXA DOS AJUSTES DA CONTA -------------*/
            SELECT
               COUNT(*)
            INTO V_NRREGFOR
            FROM
               AJCTRECEBER
            WHERE
               (CDFILIAL     = C_CONTA.CDFILIAL)     AND
               (NRLANCTONF   = C_CONTA.NRLANCTONF)   AND
               (NRSEQVENREC  = C_CONTA.NRSEQVENREC)  AND
               (NRPARCELAREC = C_CONTA.NRPARCELAREC);

            IF V_NRREGFOR > 0 THEN
               FOR C_AJUSTE IN
                  (SELECT
                      AJ.CDFILIAL,     AJ.NRLANCTONF,   AJ.NRSEQVENREC,
                      AJ.NRPARCELAREC, AJ.DTAJUSTEREC,  AJ.CDAJUCTREC,
                      AJ.CDCLASSFINA,  AJ.CDEMMFAJCNTR, AJ.CDFIMFAJCNTR,
                      AJ.DTMOVFLUXO,   AJ.IDMOVFLUXO,   AJ.VRAJUSTEREC,
                      TA.IDAJUCTREC
                   FROM
                      TIPOAJRECEBER TA, AJCTRECEBER   AJ
                   WHERE
                      (AJ.CDFILIAL     = C_CONTA.CDFILIAL)     AND
                      (AJ.NRLANCTONF   = C_CONTA.NRLANCTONF)   AND
                      (AJ.NRSEQVENREC  = C_CONTA.NRSEQVENREC)  AND
                      (AJ.NRPARCELAREC = C_CONTA.NRPARCELAREC) AND
                      (TA.CDAJUCTREC   = AJ.CDAJUCTREC)
                   ) LOOP

                  IF C_AJUSTE.IDAJUCTREC = 'A' THEN
                     ATUALIZA_FLUXO_DE_CAIXA(C_AJUSTE.CDEMMFAJCNTR,
                                             C_AJUSTE.CDFIMFAJCNTR,
                                             C_AJUSTE.DTMOVFLUXO,
                                             C_AJUSTE.IDMOVFLUXO,
                                             C_AJUSTE.CDCLASSFINA,
                                             C_AJUSTE.VRAJUSTEREC * -1,
                                             0);

                     ATUALIZA_FLUXO_DE_CAIXA(C_AJUSTE.CDEMMFAJCNTR,
                                             C_AJUSTE.CDFIMFAJCNTR,
                                             V_DTVENCDUP + TRUNC(C_CONTA.QTDIASPREREC),
                                             C_AJUSTE.IDMOVFLUXO,
                                             C_AJUSTE.CDCLASSFINA,
                                             C_AJUSTE.VRAJUSTEREC,
                                             0);
                  ELSE
                     ATUALIZA_FLUXO_DE_CAIXA(C_AJUSTE.CDEMMFAJCNTR,
                                             C_AJUSTE.CDFIMFAJCNTR,
                                             C_AJUSTE.DTMOVFLUXO,
                                             C_AJUSTE.IDMOVFLUXO,
                                             C_AJUSTE.CDCLASSFINA,
                                             C_AJUSTE.VRAJUSTEREC,
                                             0);

                     ATUALIZA_FLUXO_DE_CAIXA(C_AJUSTE.CDEMMFAJCNTR,
                                             C_AJUSTE.CDFIMFAJCNTR,
                                             V_DTVENCDUP + TRUNC(C_CONTA.QTDIASPREREC),
                                             C_AJUSTE.IDMOVFLUXO,
                                             C_AJUSTE.CDCLASSFINA,
                                             C_AJUSTE.VRAJUSTEREC * -1,
                                             0);
                  END IF;

                  UPDATE AJCTRECEBER
                  SET DTMOVFLUXO = V_DTVENCDUP + TRUNC(C_CONTA.QTDIASPREREC)
                  WHERE
                     (CDFILIAL     = C_AJUSTE.CDFILIAL)     AND
                     (NRLANCTONF   = C_AJUSTE.NRLANCTONF)   AND
                     (NRSEQVENREC  = C_AJUSTE.NRSEQVENREC)  AND
                     (NRPARCELAREC = C_AJUSTE.NRPARCELAREC) AND
                     (DTAJUSTEREC  = C_AJUSTE.DTAJUSTEREC)  AND
                     (CDAJUCTREC   = C_AJUSTE.CDAJUCTREC)   AND
                     (CDCLASSFINA  = C_AJUSTE.CDCLASSFINA);
               END LOOP;
            END IF;
         END IF;

         UPDATE CONTARECEBER
         SET CDFILIBOLREC = NULL,
             NRLANBOLREC  = NULL
         WHERE
            (CDFILIBOLREC = C_CONTA.CDFILIBOLREC) AND
            (NRLANBOLREC  = C_CONTA.NRLANBOLREC);

         /* -- EXCLUI OS AJUSTES DOS BOLETOS E DESFAZ O FLUXO DE CAIXA -------*/
         SELECT
            COUNT(*)
         INTO
            V_CTAJBOL
         FROM
            AJBOLREC
         WHERE
            (CDFILIAL    = C_CONTA.CDFILIBOLREC) AND
            (NRLANBOLREC = C_CONTA.NRLANBOLREC);

         IF V_CTAJBOL > 0 THEN
            FOR C_AJBOLREC IN
               (SELECT
                   TA.IDAJUCTREC,  MA.CDCLASSFINA, MA.CDEMPRESA,
                   MA.CDFILIAL,    MA.DTMOVFLUXO,  MA.IDMOVFLUXO,
                   MA.VRFXAJBOLREC
                FROM
                   TIPOAJRECEBER TA, MOVFXAJBOLR MA
                WHERE
                   (MA.CDFILIBOLREC = C_CONTA.CDFILIBOLREC) AND
                   (MA.NRLANBOLREC  = C_CONTA.NRLANBOLREC)  AND
                   (TA.CDAJUCTREC   = MA.CDAJUCTREC)
                ) LOOP

               IF C_AJBOLREC.IDAJUCTREC = 'A' THEN
                  ATUALIZA_FLUXO_DE_CAIXA(C_AJBOLREC.CDEMPRESA,
                                          C_AJBOLREC.CDFILIAL,
                                          C_AJBOLREC.DTMOVFLUXO,
                                          C_AJBOLREC.IDMOVFLUXO,
                                          C_AJBOLREC.CDCLASSFINA,
                                          C_AJBOLREC.VRFXAJBOLREC * -1,
                                          0);
               ELSE
                  ATUALIZA_FLUXO_DE_CAIXA(C_AJBOLREC.CDEMPRESA,
                                          C_AJBOLREC.CDFILIAL,
                                          C_AJBOLREC.DTMOVFLUXO,
                                          C_AJBOLREC.IDMOVFLUXO,
                                          C_AJBOLREC.CDCLASSFINA,
                                          C_AJBOLREC.VRFXAJBOLREC,
                                          0);
               END IF;
            END LOOP;

            DELETE FROM MOVFXAJBOLR
            WHERE
               (CDFILIBOLREC = C_CONTA.CDFILIBOLREC) AND
               (NRLANBOLREC  = C_CONTA.NRLANBOLREC);

            DELETE FROM AJBOLREC
            WHERE
               (CDFILIAL    = C_CONTA.CDFILIBOLREC) AND
               (NRLANBOLREC = C_CONTA.NRLANBOLREC);
         END IF;

         DELETE FROM BOLETRECEBER
         WHERE
            (CDFILIAL    = C_CONTA.CDFILIBOLREC) AND
            (NRLANBOLREC = C_CONTA.NRLANBOLREC);
      END LOOP;

      /* -- INCLUI O NOVO BOLETO ---------------------------------------------*/
      V_PEMULTA := 0;
      V_PEMORA  := 0;

      SELECT
         MAX(CL.CDCLIENTE),
         MAX(CL.VRPEMULBCLI),
         MAX(CL.VRPEMORBCLI)
      INTO
         V_CDCLIENTE,
         V_PEMULTA,
         V_PEMORA
      FROM
         CONTARECEBER CR,
         NOTAFISCAL   NF,
         CLIENTE      CL
      WHERE
         (CR.CDFILIALDUP = P_CDFILIAL)    AND
         (CR.NRLANCTODUP = P_NRLANCTODUP) AND
         (NF.CDFILIAL    = CR.CDFILIAL)   AND
         (NF.NRLANCTONF  = CR.NRLANCTONF) AND
         (CL.CDCLIENTE   = NF.CDCLIENTE);

      IF (V_PEMULTA IS NULL) OR
         (V_PEMORA  IS NULL) THEN
         SELECT
            MAX(VRPEMULEMPR),
            MAX(VRPEMOREMPR)
         INTO
            V_PEMULTA,
            V_PEMORA
         FROM
            EMPRESA
         WHERE
            (CDEMPRESA = V_CDEMPRESA);
      END IF;

      IF (NOT V_CDREDUCONT IS NULL) THEN
         SELECT MAX(TXMENSBOLETO)
         INTO V_TXMENSBOL
         FROM
            EMPRBANC EB,
            CONTCORR CC
         WHERE
            CC.CDREDUCONT = V_CDREDUCONT AND
            EB.CDEMPRESA  = V_CDEMPRESA  AND
            EB.CDBANCO    = CC.CDBANCO;

         IF (NOT V_TXMENSBOL IS NULL) THEN
            IF LENGTH(V_TXMENSBOL ||' '||V_NRNOTFISC) < 500 THEN
               V_NRNOTFISC := V_TXMENSBOL ||' '||V_NRNOTFISC;
            ELSE
               V_NRNOTFISC := V_TXMENSBOL;
            END IF;
         END IF;
      END IF;

      IF LENGTH(V_NRNOTFISC) > 500 THEN
         V_NRNOTFISC := '';
      END IF;

      V_TIPOBAIXA := 0;

      SELECT
         COUNT(*)
      INTO
         V_TIPOBAIXA
      FROM
         CLIENTE CL, TIPOBAIXAREC TP
      WHERE
             (TP.CDTPBAIXAREC = CL.CDTPBAIXAREC   )
         AND (CL.CDCLIENTE    = V_CDCLIENTE       )
      ORDER BY
         CL.NMRAZSOCCLIE;

      IF V_TIPOBAIXA > 0 THEN
         SELECT
            CL.CDTPBAIXAREC, CL.CDCLIENTE,
            CL.NMRAZSOCCLIE, CL.NMFANTCLIE , TP.NMTPBAIXAREC
         INTO
            V_CDTPBAIXA, V_CDCLIENTE,
            V_NMRAZSOCCLIE, V_NMFANTCLIE , V_NMTPBAIXAREC
         FROM
            CLIENTE CL, TIPOBAIXAREC TP
         WHERE
                (TP.CDTPBAIXAREC = CL.CDTPBAIXAREC   )
            AND (CL.CDCLIENTE    = V_CDCLIENTE       )
         ORDER BY
            CL.NMRAZSOCCLIE;


         IF NOT V_CDTPBAIXA IS NULL THEN
            V_CDTPBAIXAREC := V_CDTPBAIXA;
         END IF ;
      END IF;


      INSERT INTO BOLETRECEBER(
         CDFILIAL,    NRLANBOLREC,  CDCLIENTE,   CDEMPRESA,
         VRBOLREC,    VRPEMULBREC,  VRPEMORBREC, VRFINALBREC,
         DTVENBOLREC, CDTPBAIXAREC, CDREDUCONT, TXMENSBOLBOL)
      VALUES(
         P_CDFILIAL,  P_NRLANBOLREC,  V_CDCLIENTE,  V_CDEMPRESA,
         V_VRBOLETO,  V_PEMULTA,      V_PEMORA,     V_VRBOLETO,
         V_DTVENCDUP, V_CDTPBAIXAREC, V_CDREDUCONT, V_NRNOTFISC);

      /* -- GRAVA A CHAVE DO NOVO BOLETO NAS CONTAS DA DUPLICATA -------------*/
      UPDATE CONTARECEBER
      SET CDFILIBOLREC = P_CDFILIAL,
          NRLANBOLREC  = P_NRLANBOLREC
      WHERE
         (CDFILIALDUP = P_CDFILIAL) AND
         (NRLANCTODUP = P_NRLANCTODUP) AND
         (CDFILIAL, NRLANCTONF) IN (SELECT CDFILIAL, NRLANCTONF
                                    FROM NOTAFISCAL
                                    WHERE IDSTATUS IN ('A', 'E'));


      /* -- GRAVA OS AJUSTES DA DUPLICATA NO BOLETO -------------*/
      SELECT COUNT(*)
      INTO V_NRAJDUPLI
      FROM AJDUPBOLET
      WHERE
          (CDFILIAL     = P_CDFILIAL)    AND
          (NRLANCTODUP  = P_NRLANCTODUP);

      IF V_NRAJDUPLI > 0 THEN
         V_VRAJDUPLI := 0;

         FOR C_AJDUPLI IN
            (SELECT
                AD.CDFILIAL,     AD.NRLANCTODUP, AD.CDAJUCTREC,
                AD.VRAJDUPBOLET, TA.IDAJUCTREC
             FROM
                TIPOAJRECEBER TA, AJDUPBOLET  AD
             WHERE
                (AD.CDFILIAL    = P_CDFILIAL)    AND
                (AD.NRLANCTODUP = P_NRLANCTODUP) AND
                (TA.CDAJUCTREC  = AD.CDAJUCTREC)
            ) LOOP

            INSERT INTO AJBOLREC
               (CDFILIAL, NRLANBOLREC,
                CDAJUCTREC, VRAJBOLREC)
            VALUES
               (P_CDFILIAL, P_NRLANBOLREC,
                C_AJDUPLI.CDAJUCTREC, C_AJDUPLI.VRAJDUPBOLET);

            IF C_AJDUPLI.IDAJUCTREC = 'A' THEN
               V_VRAJDUPLI := V_VRAJDUPLI + C_AJDUPLI.VRAJDUPBOLET;
            ELSE
               V_VRAJDUPLI := V_VRAJDUPLI - C_AJDUPLI.VRAJDUPBOLET;
            END IF;
         END LOOP;

         /* -- ALTERA O VALOR DO BOLETO -------------*/
         UPDATE BOLETRECEBER
         SET VRFINALBREC = VRFINALBREC + V_VRAJDUPLI
         WHERE
            (CDFILIAL    = P_CDFILIAL)    AND
            (NRLANBOLREC = P_NRLANBOLREC);


         /* -- ATUALIZA FLUXO DE CAIXA DO AJUSTE -------------*/
         FOR C_AJBOLFX IN
            (SELECT
                AJ.CDFILIAL,   AJ.NRLANBOLREC,  AJ.CDAJUCTREC,
                AJ.VRAJBOLREC, TA.IDAJUCTREC,   CR.CDCLASSFINA,
                CR.CDEMPRESA,  CR.CDFIMOVFLREC, CR.DTMOVFLUXO,
                CR.IDMOVFLUXO, SUM(CR.VRATUAREC) VRATUAREC,
                NVL(CR.QTDIASPREREC, 0) AS QTDIASPREREC
             FROM
                TIPOAJRECEBER  TA, CONTARECEBER CR, AJBOLREC AJ
             WHERE
                (AJ.CDFILIAL    = P_CDFILIAL)      AND
                (AJ.NRLANBOLREC = P_NRLANBOLREC)   AND
                (AJ.CDFILIAL    = CR.CDFILIBOLREC) AND
                (AJ.NRLANBOLREC = CR.NRLANBOLREC)  AND
                (TA.CDAJUCTREC  = AJ.CDAJUCTREC)
             GROUP BY
                AJ.CDFILIAL,   AJ.NRLANBOLREC,  AJ.CDAJUCTREC,
                AJ.VRAJBOLREC, TA.IDAJUCTREC,   CR.CDCLASSFINA,
                CR.CDEMPRESA,  CR.CDFIMOVFLREC, CR.DTMOVFLUXO,
                CR.IDMOVFLUXO, CR.QTDIASPREREC
            ) LOOP

            V_VRMOVFXBOL := 0;
            IF V_VRBOLETO IS NOT NULL THEN
               V_VRMOVFXBOL := ROUND((C_AJBOLFX.VRATUAREC * C_AJBOLFX.VRAJBOLREC)/V_VRBOLETO,2);
            END IF;

            IF C_AJBOLFX.IDAJUCTREC = 'A' THEN
               ATUALIZA_FLUXO_DE_CAIXA(V_CDEMPRESA,
                                       C_AJBOLFX.CDFIMOVFLREC,
                                       V_DTVENCDUP + TRUNC(C_AJBOLFX.QTDIASPREREC),
                                       'E',
                                       C_AJBOLFX.CDCLASSFINA,
                                       V_VRMOVFXBOL,
                                       0);
            ELSE
               ATUALIZA_FLUXO_DE_CAIXA(V_CDEMPRESA,
                                       C_AJBOLFX.CDFIMOVFLREC,
                                       V_DTVENCDUP + TRUNC(C_AJBOLFX.QTDIASPREREC),
                                       'E',
                                       C_AJBOLFX.CDCLASSFINA,
                                       V_VRMOVFXBOL * -1,
                                       0);
            END IF;

            INSERT INTO MOVFXAJBOLR
               (CDEMPRESA,  CDFILIAL, DTMOVFLUXO,
                CDCLASSFINA,  IDMOVFLUXO, CDFILIBOLREC,
                NRLANBOLREC, CDAJUCTREC,  VRFXAJBOLREC)
            VALUES
                (V_CDEMPRESA, C_AJBOLFX.CDFIMOVFLREC, V_DTVENCDUP + TRUNC(C_AJBOLFX.QTDIASPREREC),
                 C_AJBOLFX.CDCLASSFINA, 'E', P_CDFILIAL,
                 P_NRLANBOLREC, C_AJBOLFX.CDAJUCTREC, V_VRMOVFXBOL);
         END LOOP;
      END IF;
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('INCLUSAO_DE_DUPLICATA','5.01.003'); END;
/

